<?php

namespace Drp\Peek;

use Carbon\Carbon;
use Drp\Peek\Blocks\Extensions;
use Drp\Peek\Blocks\PhpSettings;
use Drp\Peek\Blocks\Versions;
use Drp\Peek\Filesystem\FilesystemManager;
use Illuminate\Contracts\Auth\Access\Gate;
use Illuminate\Contracts\Foundation\Application;
use Illuminate\Support\Facades\Storage;
use Illuminate\Support\ServiceProvider;

class PeekServiceProvider extends ServiceProvider
{
    /**
     * Bootstrap the application services.
     */
    public function boot(): void
    {
        $this->loadViewsFrom(__DIR__.'/../resources/views', 'peek');

        $this->loadRoutes();
        $this->registerFilesystems();
        $this->app->instance('peek:logo', 'peek::partials.logos.' . $this->decideLogo());

        if (!$this->app->runningInConsole()) {
            return;
        }

        $this->publishes([
            __DIR__.'/../config/config.php' => config_path('peek.php'),
        ], 'config');

        $this->publishes([
            __DIR__.'/../dist' => public_path('vendor/peek'),
        ], 'public');
    }

    /**
     * Register the application services.
     */
    public function register(): void
    {
        // Automatically apply the package configuration
        $this->mergeConfigFrom(__DIR__.'/../config/config.php', 'peek');

        // Register the main class to use with the facade
        $this->app->singleton('peek', function ($app) {
            return new Peek($app[Gate::class]);
        });

        $this->app->bind(Versions::class, function (Application $app) {
            return new Versions(phpversion(), $app->version());
        });

        $this->app->bind(PhpSettings::class, function () {
            return new PhpSettings(
                ini_get('memory_limit'),
                ini_get('upload_max_filesize'),
                ini_get('max_file_uploads'),
                ini_get('max_execution_time')
            );
        });

        $this->app->bind(Extensions::class, function () {
            $extensions = array_map(function (string $extensionName) {
                return (object) [
                    'name' => $extensionName,
                    'version' => phpversion($extensionName),
                ];
            }, get_loaded_extensions());

            return new Extensions($extensions);
        });

        $this->applyLaravel51Fixes();
    }

    private function loadRoutes(): void
    {
        if ($this->app->version() < '5.2') {
            require __DIR__ . '/routes.php';
            return;
        }

        $this->loadRoutesFrom(__DIR__ . '/routes.php');
    }

    private function registerFilesystems(): void
    {
        Storage::set(
            'peek:root',
            Storage::createLocalDriver(['root' => base_path()])
        );

        Storage::set(
            'peek:commands',
            Storage::createLocalDriver(['root' => storage_path('app/peek')])
        );

        Storage::set(
            'peek:logs',
            Storage::createLocalDriver(['root' => storage_path('logs')])
        );
    }

    private function applyLaravel51Fixes(): void
    {
        if ($this->app->version() >= '5.2') {
            return;
        }

        $this->app->singleton(
            'filesystem',
            function ($app) {
                return new FilesystemManager($app);
            }
        );
    }

    private function decideLogo(): string
    {
        if (Carbon::now()->gt(Carbon::today()->setTime(20, 0)) || Carbon::now()->lt(Carbon::today()->setTime(6, 0))) {
            return 'popped';
        }

        if (session()->has('success')) {
            return 'pleased';
        }

        return 'passive';
    }
}
